@@ -44,8 +44,7 @@
 #include <nettle/ecc.h>
 #include <nettle/ecdsa.h>
 #include <nettle/ecc-curve.h>
-
-#define TOMPZ(x) (*((mpz_t*)(x)))
+#include <gnettle.h>
 
 static inline const struct ecc_curve *get_supported_curve(int curve);
 
@@ -58,29 +57,29 @@ static void
 _dsa_params_to_pubkey(const gnutls_pk_params_st * pk_params,
 		      struct dsa_public_key *pub)
 {
-	memcpy(&pub->p, pk_params->params[0], sizeof(mpz_t));
-	memcpy(&pub->q, pk_params->params[1], sizeof(mpz_t));
-	memcpy(&pub->g, pk_params->params[2], sizeof(mpz_t));
-	memcpy(&pub->y, pk_params->params[3], sizeof(mpz_t));
+	memcpy(pub->p, pk_params->params[0], SIZEOF_MPZT);
+	memcpy(pub->q, pk_params->params[1], SIZEOF_MPZT);
+	memcpy(pub->g, pk_params->params[2], SIZEOF_MPZT);
+	memcpy(pub->y, pk_params->params[3], SIZEOF_MPZT);
 }
 
 static void
 _dsa_params_to_privkey(const gnutls_pk_params_st * pk_params,
 		       struct dsa_private_key *pub)
 {
-	memcpy(&pub->x, pk_params->params[4], sizeof(mpz_t));
+	memcpy(pub->x, pk_params->params[4], SIZEOF_MPZT);
 }
 
 static void
 _rsa_params_to_privkey(const gnutls_pk_params_st * pk_params,
 		       struct rsa_private_key *priv)
 {
-	memcpy(&priv->d, pk_params->params[2], sizeof(mpz_t));
-	memcpy(&priv->p, pk_params->params[3], sizeof(mpz_t));
-	memcpy(&priv->q, pk_params->params[4], sizeof(mpz_t));
-	memcpy(&priv->c, pk_params->params[5], sizeof(mpz_t));
-	memcpy(&priv->a, pk_params->params[6], sizeof(mpz_t));
-	memcpy(&priv->b, pk_params->params[7], sizeof(mpz_t));
+	memcpy(priv->d, pk_params->params[2], SIZEOF_MPZT);
+	memcpy(priv->p, pk_params->params[3], SIZEOF_MPZT);
+	memcpy(priv->q, pk_params->params[4], SIZEOF_MPZT);
+	memcpy(priv->c, pk_params->params[5], SIZEOF_MPZT);
+	memcpy(priv->a, pk_params->params[6], SIZEOF_MPZT);
+	memcpy(priv->b, pk_params->params[7], SIZEOF_MPZT);
 	priv->size =
 	    nettle_mpz_sizeinbase_256_u(TOMPZ
 					(pk_params->params[RSA_MODULUS]));
@@ -90,8 +89,8 @@ static void
 _rsa_params_to_pubkey(const gnutls_pk_params_st * pk_params,
 		      struct rsa_public_key *pub)
 {
-	memcpy(&pub->n, pk_params->params[RSA_MODULUS], sizeof(mpz_t));
-	memcpy(&pub->e, pk_params->params[RSA_PUB], sizeof(mpz_t));
+	memcpy(pub->n, pk_params->params[RSA_MODULUS], SIZEOF_MPZT);
+	memcpy(pub->e, pk_params->params[RSA_PUB], SIZEOF_MPZT);
 	pub->size = nettle_mpz_sizeinbase_256_u(pub->n);
 }
 
@@ -526,8 +525,8 @@ _wrap_nettle_pk_verify(gnutls_pk_algorithm_t algo,
 				goto cleanup;
 			}
 
-			memcpy(&sig.r, tmp[0], sizeof(sig.r));
-			memcpy(&sig.s, tmp[1], sizeof(sig.s));
+			memcpy(sig.r, tmp[0], SIZEOF_MPZT);
+			memcpy(sig.s, tmp[1], SIZEOF_MPZT);
 
 			_gnutls_dsa_q_to_hash(algo, pk_params, &hash_len);
 
@@ -560,8 +559,8 @@ _wrap_nettle_pk_verify(gnutls_pk_algorithm_t algo,
 			}
 			memset(&pub, 0, sizeof(pub));
 			_dsa_params_to_pubkey(pk_params, &pub);
-			memcpy(&sig.r, tmp[0], sizeof(sig.r));
-			memcpy(&sig.s, tmp[1], sizeof(sig.s));
+			memcpy(sig.r, tmp[0], SIZEOF_MPZT);
+			memcpy(sig.s, tmp[1], SIZEOF_MPZT);
 
 			_gnutls_dsa_q_to_hash(algo, pk_params, &hash_len);
 
