@@ -44,6 +44,8 @@
 
 struct gnutls_pkcs11_provider_s {
 	struct ck_function_list *module;
+	unsigned trusted; /* in the sense of p11-kit trusted:
+	                   * it can be used for verification */
 	struct ck_info info;
 	unsigned int initialized;
 };
@@ -51,6 +53,7 @@ struct gnutls_pkcs11_provider_s {
 struct find_flags_data_st {
 	struct p11_kit_uri *info;
 	unsigned int slot_flags;
+	unsigned int trusted;
 };
 
 struct find_url_data_st {
@@ -220,6 +223,9 @@ pkcs11_add_module(const char *name, struct ck_function_list *module)
 	active_providers++;
 	providers[active_providers - 1].module = module;
 
+	if (p11_kit_module_get_flags(module) & P11_KIT_MODULE_TRUSTED)
+		providers[active_providers - 1].trusted = 1;
+
 	memcpy(&providers[active_providers - 1].info, &info, sizeof(info));
 
 	_gnutls_debug_log("p11: loaded provider '%s'\n", name);
@@ -703,7 +709,8 @@ int gnutls_pkcs11_obj_init(gnutls_pkcs11_obj_t * obj)
 
 	(*obj)->info = p11_kit_uri_new();
 	if ((*obj)->info == NULL) {
-		free(*obj);
+		gnutls_free(*obj);
+		*obj = NULL;
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
@@ -1004,6 +1011,9 @@ _pkcs11_traverse_tokens(find_func_t find_func, void *input,
 
 	for (x = 0; x < active_providers; x++) {
 
+		if (flags & SESSION_TRUSTED && providers[x].trusted == 0)
+			continue;
+
 		nslots = sizeof(slots) / sizeof(slots[0]);
 		ret = scan_slots(&providers[x], slots, &nslots);
 		if (ret < 0) {
@@ -1016,8 +1026,7 @@ _pkcs11_traverse_tokens(find_func_t find_func, void *input,
 			struct token_info tinfo;
 
 			if (pkcs11_get_token_info(module, slots[z],
-						  &tinfo.tinfo) != CKR_OK)
-			{
+						  &tinfo.tinfo) != CKR_OK) {
 				continue;
 			}
 			tinfo.sid = slots[z];
@@ -1575,6 +1584,8 @@ unsigned int pkcs11_obj_flags_to_int(unsigned int flags)
 		ret_flags |= SESSION_LOGIN;
 	if (flags & GNUTLS_PKCS11_OBJ_FLAG_LOGIN_SO)
 		ret_flags |= SESSION_LOGIN | SESSION_SO;
+	if (flags & GNUTLS_PKCS11_OBJ_FLAG_PRESENT_IN_TRUSTED_MODULE)
+		ret_flags |= SESSION_TRUSTED;
 
 	return ret_flags;
 }
@@ -1763,8 +1774,10 @@ gnutls_pkcs11_token_get_info(const char *url,
 
 	*output_size = len;
 
+	ret = 0;
+
 	p11_kit_uri_free(info);
-	return 0;
+	return ret;
 }
 
 /**
@@ -2417,8 +2430,11 @@ find_objs(struct pkcs11_session_info *sinfo,
 			a[0].value = &class;
 			a[0].value_len = sizeof class;
 
-			pkcs11_get_attribute_value(sinfo->module,
+			rv = pkcs11_get_attribute_value(sinfo->module,
 						   sinfo->pks, obj, a, 1);
+			if (rv != CKR_OK) {
+				class = -1;
+			}
 		}
 
 		if (find_data->flags ==
@@ -2760,7 +2776,10 @@ find_flags(struct pkcs11_session_info *sinfo,
 	}
 
 	/* found token! */
-
+	if (p11_kit_module_get_flags(sinfo->module) & P11_KIT_MODULE_TRUSTED)
+		find_data->trusted = 1;
+	else
+		find_data->trusted = 0;
 	find_data->slot_flags = info->sinfo.flags;
 
 	return 0;
@@ -2772,7 +2791,8 @@ find_flags(struct pkcs11_session_info *sinfo,
  * @flags: The output flags (GNUTLS_PKCS11_TOKEN_*)
  *
  * This function will return information about the PKCS 11 token flags.
- * The flags from the %gnutls_pkcs11_token_info_t enumeration.
+ *
+ * The supported flags are: %GNUTLS_PKCS11_TOKEN_HW and %GNUTLS_PKCS11_TOKEN_TRUSTED.
  *
  * Returns: %GNUTLS_E_SUCCESS (0) on success or a negative error code on error.
  *
@@ -2804,6 +2824,9 @@ int gnutls_pkcs11_token_get_flags(const char *url, unsigned int *flags)
 	if (find_data.slot_flags & CKF_HW_SLOT)
 		*flags |= GNUTLS_PKCS11_TOKEN_HW;
 
+	if (find_data.trusted != 0)
+		*flags |= GNUTLS_PKCS11_TOKEN_TRUSTED;
+
 	return 0;
 
 }
@@ -2927,6 +2950,14 @@ find_cert(struct pkcs11_session_info *sinfo,
 		return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
 	}
 
+	/* the DISTRUSTED flag is p11-kit module specific */
+	if (priv->flags & GNUTLS_PKCS11_OBJ_FLAG_RETRIEVE_DISTRUSTED) {
+		if (memcmp(lib_info->manufacturer_id, "PKCS#11 Kit", 11) != 0) {
+			gnutls_assert();
+			return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
+		}
+	}
+
 	if (priv->dn.size == 0 && priv->key_id.size == 0 && priv->issuer_dn.size == 0 &&
 		priv->serial.size == 0)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
@@ -3200,9 +3231,17 @@ int gnutls_pkcs11_get_raw_issuer(const char *url, gnutls_x509_crt_t cert,
  *
  * This function will check whether the provided certificate is stored
  * in the specified token. This is useful in combination with 
- * %GNUTLS_PKCS11_OBJ_FLAG_RETRIEVE_TRUSTED or %GNUTLS_PKCS11_OBJ_FLAG_RETRIEVE_TRUSTED,
+ * %GNUTLS_PKCS11_OBJ_FLAG_RETRIEVE_TRUSTED or 
+ * %GNUTLS_PKCS11_OBJ_FLAG_RETRIEVE_DISTRUSTED,
  * to check whether a CA is present or a certificate is blacklisted in
- * trust PKCS #11 modules.
+ * a trust PKCS #11 module.
+ *
+ * This function can be used with a @url of "pkcs11:", and in that case all modules
+ * will be searched. To restrict the modules to the marked as trusted in p11-kit
+ * use the %GNUTLS_PKCS11_OBJ_FLAG_PRESENT_IN_TRUSTED_MODULE flag.
+ *
+ * Note that the flag %GNUTLS_PKCS11_OBJ_FLAG_RETRIEVE_DISTRUSTED is
+ * specific to p11-kit trust modules.
  *
  * Returns: If the certificate exists non-zero is returned, otherwise zero.
  *
@@ -3231,6 +3270,7 @@ int _gnutls_pkcs11_crt_is_known(const char *url, gnutls_x509_crt_t cert,
 		return 0;
 	}
 
+	/* Attempt searching using the issuer DN + serial number */
 	serial_size = sizeof(serial) - sizeof(tag);
 	ret =
 	    gnutls_x509_crt_get_serial(cert, serial+sizeof(tag), &serial_size);
@@ -3267,6 +3307,18 @@ int _gnutls_pkcs11_crt_is_known(const char *url, gnutls_x509_crt_t cert,
 	ret =
 	    _pkcs11_traverse_tokens(find_cert, &priv, info,
 				    NULL, pkcs11_obj_flags_to_int(flags));
+	if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
+		/* attempt searching with the subject DN only */
+		memset(&priv, 0, sizeof(priv));
+		priv.crt = cert;
+		priv.flags = flags;
+
+		priv.dn.data = cert->raw_dn.data;
+		priv.dn.size = cert->raw_dn.size;
+		ret =
+		    _pkcs11_traverse_tokens(find_cert, &priv, info,
+				    NULL, pkcs11_obj_flags_to_int(flags));
+	}
 	if (ret < 0) {
 		gnutls_assert();
 		ret = 0;
