@@ -69,7 +69,7 @@ FILE *safe_open_rw(const char *file, int privkey_op)
 
 gnutls_datum_t *load_secret_key(int mand, common_info_st * info)
 {
-	char raw_key[64];
+	static char raw_key[64];
 	size_t raw_key_size = sizeof(raw_key);
 	static gnutls_datum_t key;
 	gnutls_datum_t hex_key;
@@ -80,7 +80,7 @@ gnutls_datum_t *load_secret_key(int mand, common_info_st * info)
 
 	if (info->secret_key == NULL) {
 		if (mand) {
-			fprintf(stderr, "missing --secret-key");
+			fprintf(stderr, "missing --secret-key\n");
 			exit(1);
 		} else
 			return NULL;
@@ -91,7 +91,7 @@ gnutls_datum_t *load_secret_key(int mand, common_info_st * info)
 
 	ret = gnutls_hex_decode(&hex_key, raw_key, &raw_key_size);
 	if (ret < 0) {
-		fprintf(stderr, "hex_decode: %s", gnutls_strerror(ret));
+		fprintf(stderr, "hex_decode: %s\n", gnutls_strerror(ret));
 		exit(1);
 	}
 
@@ -104,20 +104,25 @@ gnutls_datum_t *load_secret_key(int mand, common_info_st * info)
 const char *get_password(common_info_st * cinfo, unsigned int *flags,
 			 int confirm)
 {
+	const char *p;
+
 	if (cinfo->null_password) {
 		if (flags)
 			*flags |= GNUTLS_PKCS_NULL_PASSWORD;
 		return NULL;
 	} else if (cinfo->password) {
-		if (cinfo->password[0] == 0 && flags)
-			*flags |= GNUTLS_PKCS_PLAIN;
-		return cinfo->password;
+		p = cinfo->password;
 	} else {
 		if (confirm)
-			return get_confirmed_pass(true);
+			p = get_confirmed_pass(true);
 		else
-			return get_pass();
+			p = get_pass();
 	}
+
+	if (p[0] == 0 && flags && !cinfo->empty_password)
+		*flags |= GNUTLS_PKCS_PLAIN;
+
+	return p;
 }
 
 static gnutls_privkey_t _load_privkey(gnutls_datum_t * dat,
@@ -130,7 +135,7 @@ static gnutls_privkey_t _load_privkey(gnutls_datum_t * dat,
 
 	ret = gnutls_privkey_init(&key);
 	if (ret < 0) {
-		fprintf(stderr, "privkey_init: %s", gnutls_strerror(ret));
+		fprintf(stderr, "privkey_init: %s\n", gnutls_strerror(ret));
 		exit(1);
 	}
 
@@ -148,12 +153,12 @@ static gnutls_privkey_t _load_privkey(gnutls_datum_t * dat,
 	if (ret == GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR) {
 		fprintf(stderr,
 			"import error: could not find a valid PEM header; "
-			"check if your key is PKCS #12 encoded");
+			"check if your key is PKCS #12 encoded\n");
 		exit(1);
 	}
 
 	if (ret < 0) {
-		fprintf(stderr, "importing --load-privkey: %s: %s",
+		fprintf(stderr, "importing --load-privkey: %s: %s\n",
 			info->privkey, gnutls_strerror(ret));
 		exit(1);
 	}
@@ -168,13 +173,13 @@ static gnutls_privkey_t _load_url_privkey(const char *url)
 
 	ret = gnutls_privkey_init(&key);
 	if (ret < 0) {
-		fprintf(stderr, "privkey_init: %s", gnutls_strerror(ret));
+		fprintf(stderr, "privkey_init: %s\n", gnutls_strerror(ret));
 		exit(1);
 	}
 
 	ret = gnutls_privkey_import_url(key, url, 0);
 	if (ret < 0) {
-		fprintf(stderr, "importing key: %s: %s",
+		fprintf(stderr, "importing key: %s: %s\n",
 			url, gnutls_strerror(ret));
 		exit(1);
 	}
@@ -218,7 +223,7 @@ gnutls_privkey_t load_private_key(int mand, common_info_st * info)
 		return NULL;
 
 	if (info->privkey == NULL) {
-		fprintf(stderr, "missing --load-privkey");
+		fprintf(stderr, "missing --load-privkey\n");
 		exit(1);
 	}
 
@@ -229,7 +234,7 @@ gnutls_privkey_t load_private_key(int mand, common_info_st * info)
 	dat.size = size;
 
 	if (!dat.data) {
-		fprintf(stderr, "reading --load-privkey: %s",
+		fprintf(stderr, "reading --load-privkey: %s\n",
 			info->privkey);
 		exit(1);
 	}
@@ -258,13 +263,13 @@ load_x509_private_key(int mand, common_info_st * info)
 		return NULL;
 
 	if (info->privkey == NULL) {
-		fprintf(stderr, "missing --load-privkey");
+		fprintf(stderr, "missing --load-privkey\n");
 		exit(1);
 	}
 
 	ret = gnutls_x509_privkey_init(&key);
 	if (ret < 0) {
-		fprintf(stderr, "privkey_init: %s", gnutls_strerror(ret));
+		fprintf(stderr, "privkey_init: %s\n", gnutls_strerror(ret));
 		exit(1);
 	}
 
@@ -272,7 +277,7 @@ load_x509_private_key(int mand, common_info_st * info)
 	dat.size = size;
 
 	if (!dat.data) {
-		fprintf(stderr, "reading --load-privkey: %s",
+		fprintf(stderr, "reading --load-privkey: %s\n",
 			info->privkey);
 		exit(1);
 	}
@@ -303,12 +308,12 @@ load_x509_private_key(int mand, common_info_st * info)
 	if (ret == GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR) {
 		fprintf(stderr,
 			"import error: could not find a valid PEM header; "
-			"check if your key is PEM encoded");
+			"check if your key is PEM encoded\n");
 		exit(1);
 	}
 
 	if (ret < 0) {
-		fprintf(stderr, "importing --load-privkey: %s: %s",
+		fprintf(stderr, "importing --load-privkey: %s: %s\n",
 			info->privkey, gnutls_strerror(ret));
 		exit(1);
 	}
@@ -352,7 +357,7 @@ gnutls_x509_crt_t *load_cert_list(int mand, size_t * crt_size,
 
 	if (info->cert == NULL) {
 		if (mand) {
-			fprintf(stderr, "missing --load-certificate");
+			fprintf(stderr, "missing --load-certificate\n");
 			exit(1);
 		} else
 			return NULL;
@@ -360,7 +365,7 @@ gnutls_x509_crt_t *load_cert_list(int mand, size_t * crt_size,
 
 	fd = fopen(info->cert, "r");
 	if (fd == NULL) {
-		fprintf(stderr, "%s", info->cert);
+		fprintf(stderr, "%s\n", info->cert);
 		exit(1);
 	}
 
@@ -375,7 +380,7 @@ gnutls_x509_crt_t *load_cert_list(int mand, size_t * crt_size,
 	for (i = 0; i < MAX_CERTS; i++) {
 		ret = gnutls_x509_crt_init(&crt[i]);
 		if (ret < 0) {
-			fprintf(stderr, "crt_init: %s",
+			fprintf(stderr, "crt_init: %s\n",
 				gnutls_strerror(ret));
 			exit(1);
 		}
@@ -389,7 +394,7 @@ gnutls_x509_crt_t *load_cert_list(int mand, size_t * crt_size,
 		if (ret < 0 && *crt_size > 0)
 			break;
 		if (ret < 0) {
-			fprintf(stderr, "crt_import: %s",
+			fprintf(stderr, "crt_import: %s\n",
 				gnutls_strerror(ret));
 			exit(1);
 		}
@@ -430,7 +435,7 @@ gnutls_x509_crq_t load_request(common_info_st * info)
 
 	ret = gnutls_x509_crq_init(&crq);
 	if (ret < 0) {
-		fprintf(stderr, "crq_init: %s", gnutls_strerror(ret));
+		fprintf(stderr, "crq_init: %s\n", gnutls_strerror(ret));
 		exit(1);
 	}
 
@@ -438,7 +443,7 @@ gnutls_x509_crq_t load_request(common_info_st * info)
 	dat.size = size;
 
 	if (!dat.data) {
-		fprintf(stderr, "reading --load-request: %s",
+		fprintf(stderr, "reading --load-request: %s\n",
 			info->request);
 		exit(1);
 	}
@@ -446,13 +451,13 @@ gnutls_x509_crq_t load_request(common_info_st * info)
 	ret = gnutls_x509_crq_import(crq, &dat, info->incert_format);
 	if (ret == GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR) {
 		fprintf(stderr,
-			"import error: could not find a valid PEM header");
+			"import error: could not find a valid PEM header\n");
 		exit(1);
 	}
 
 	free(dat.data);
 	if (ret < 0) {
-		fprintf(stderr, "importing --load-request: %s: %s",
+		fprintf(stderr, "importing --load-request: %s: %s\n",
 			info->request, gnutls_strerror(ret));
 		exit(1);
 	}
@@ -468,7 +473,7 @@ gnutls_privkey_t load_ca_private_key(common_info_st * info)
 	size_t size;
 
 	if (info->ca_privkey == NULL) {
-		fprintf(stderr, "missing --load-ca-privkey");
+		fprintf(stderr, "missing --load-ca-privkey\n");
 		exit(1);
 	}
 
@@ -479,7 +484,7 @@ gnutls_privkey_t load_ca_private_key(common_info_st * info)
 	dat.size = size;
 
 	if (!dat.data) {
-		fprintf(stderr, "reading --load-ca-privkey: %s",
+		fprintf(stderr, "reading --load-ca-privkey: %s\n",
 			info->ca_privkey);
 		exit(1);
 	}
@@ -501,13 +506,13 @@ gnutls_x509_crt_t load_ca_cert(common_info_st * info)
 	size_t size;
 
 	if (info->ca == NULL) {
-		fprintf(stderr, "missing --load-ca-certificate");
+		fprintf(stderr, "missing --load-ca-certificate\n");
 		exit(1);
 	}
 
 	ret = gnutls_x509_crt_init(&crt);
 	if (ret < 0) {
-		fprintf(stderr, "crt_init: %s", gnutls_strerror(ret));
+		fprintf(stderr, "crt_init: %s\n", gnutls_strerror(ret));
 		exit(1);
 	}
 
@@ -515,7 +520,7 @@ gnutls_x509_crt_t load_ca_cert(common_info_st * info)
 	dat.size = size;
 
 	if (!dat.data) {
-		fprintf(stderr, "reading --load-ca-certificate: %s",
+		fprintf(stderr, "reading --load-ca-certificate: %s\n",
 			info->ca);
 		exit(1);
 	}
@@ -523,7 +528,7 @@ gnutls_x509_crt_t load_ca_cert(common_info_st * info)
 	ret = gnutls_x509_crt_import(crt, &dat, info->incert_format);
 	free(dat.data);
 	if (ret < 0) {
-		fprintf(stderr, "importing --load-ca-certificate: %s: %s",
+		fprintf(stderr, "importing --load-ca-certificate: %s: %s\n",
 			info->ca, gnutls_strerror(ret));
 		exit(1);
 	}
@@ -545,7 +550,7 @@ gnutls_pubkey_t load_pubkey(int mand, common_info_st * info)
 		return NULL;
 
 	if (info->pubkey == NULL) {
-		fprintf(stderr, "missing --load-pubkey");
+		fprintf(stderr, "missing --load-pubkey\n");
 		exit(1);
 	}
 
@@ -554,7 +559,7 @@ gnutls_pubkey_t load_pubkey(int mand, common_info_st * info)
 
 	ret = gnutls_pubkey_init(&key);
 	if (ret < 0) {
-		fprintf(stderr, "privkey_init: %s", gnutls_strerror(ret));
+		fprintf(stderr, "privkey_init: %s\n", gnutls_strerror(ret));
 		exit(1);
 	}
 
@@ -562,7 +567,7 @@ gnutls_pubkey_t load_pubkey(int mand, common_info_st * info)
 	dat.size = size;
 
 	if (!dat.data) {
-		fprintf(stderr, "reading --load-pubkey: %s", info->pubkey);
+		fprintf(stderr, "reading --load-pubkey: %s\n", info->pubkey);
 		exit(1);
 	}
 
@@ -573,12 +578,12 @@ gnutls_pubkey_t load_pubkey(int mand, common_info_st * info)
 	if (ret == GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR) {
 		fprintf(stderr,
 			"import error: could not find a valid PEM header; "
-			"check if your key has the PUBLIC KEY header");
+			"check if your key has the PUBLIC KEY header\n");
 		exit(1);
 	}
 
 	if (ret < 0) {
-		fprintf(stderr, "importing --load-pubkey: %s: %s",
+		fprintf(stderr, "importing --load-pubkey: %s: %s\n",
 			info->pubkey, gnutls_strerror(ret));
 		exit(1);
 	}
@@ -595,7 +600,7 @@ gnutls_pubkey_t load_public_key_or_import(int mand,
 
 	ret = gnutls_pubkey_init(&pubkey);
 	if (ret < 0) {
-		fprintf(stderr, "gnutls_pubkey_init: %s",
+		fprintf(stderr, "gnutls_pubkey_init: %s\n",
 			gnutls_strerror(ret));
 		exit(1);
 	}
@@ -624,15 +629,17 @@ get_bits(gnutls_pk_algorithm_t key_type, int info_bits,
 		}
 		bits = info_bits;
 	} else {
-		if (info_sec_param) {
-			bits =
-			    gnutls_sec_param_to_pk_bits(key_type,
-							str_to_sec_param
-							(info_sec_param));
-		} else
-			bits =
-			    gnutls_sec_param_to_pk_bits(key_type,
-							GNUTLS_SEC_PARAM_NORMAL);
+		if (info_sec_param == 0) {
+			/* For ECDSA keys use 256 bits or better, as they are widely supported */
+			if (key_type == GNUTLS_PK_EC)
+				info_sec_param = "HIGH";
+			else
+				info_sec_param = "NORMAL";
+		}
+		bits =
+		    gnutls_sec_param_to_pk_bits(key_type,
+						str_to_sec_param
+						(info_sec_param));
 	}
 
 	return bits;
@@ -795,7 +802,7 @@ void _pubkey_info(FILE * outfile,
 
 	ret = gnutls_pubkey_print(pubkey, format, &data);
 	if (ret < 0) {
-		fprintf(stderr, "pubkey_print error: %s",
+		fprintf(stderr, "pubkey_print error: %s\n",
 			gnutls_strerror(ret));
 		exit(1);
 	}
@@ -808,7 +815,7 @@ void _pubkey_info(FILE * outfile,
 	    gnutls_pubkey_export(pubkey, GNUTLS_X509_FMT_PEM, buffer,
 				 &size);
 	if (ret < 0) {
-		fprintf(stderr, "export error: %s", gnutls_strerror(ret));
+		fprintf(stderr, "export error: %s\n", gnutls_strerror(ret));
 		exit(1);
 	}
 
