@@ -45,6 +45,7 @@
 #include <gnutls/x509.h>
 #include <gnutls/openpgp.h>
 #include <gnutls/pkcs11.h>
+#include <gnutls/crypto.h>
 
 /* Gnulib portability files. */
 #include <read-file.h>
@@ -420,19 +421,24 @@ static int cert_verify_callback(gnutls_session_t session)
 	int rc;
 	unsigned int status = 0;
 	int ssh = ENABLED_OPT(TOFU);
+	int strictssh = ENABLED_OPT(STRICT_TOFU);
 #ifdef HAVE_DANE
 	int dane = ENABLED_OPT(DANE);
 #endif
 	int ca_verify = ENABLED_OPT(CA_VERIFICATION);
 	const char *txt_service;
 
+	if (strictssh) {
+		ssh = strictssh;
+	}
+
 	print_cert_info(session, verbose, print_cert);
 
 	if (ca_verify) {
 		rc = cert_verify(session, hostname);
 		if (rc == 0) {
 			printf
-			    ("*** Verifying server certificate failed...\n");
+			    ("*** PKI verification of server certificate failed...\n");
 			if (!insecure && !ssh)
 				return -1;
 		} else if (ENABLED_OPT(OCSP) && gnutls_ocsp_status_request_is_checked(session, 0) == 0) {	/* off-line verification succeeded. Try OCSP */
@@ -443,10 +449,47 @@ static int cert_verify_callback(gnutls_session_t session)
 				if (!insecure && !ssh)
 					return -1;
 			} else if (rc == -1)
-				printf("*** OCSP response ignored\n");
+				printf("*** OCSP response ignored...\n");
 		}
 	}
 
+#ifdef HAVE_DANE
+	if (dane) {		/* try DANE auth */
+		int port;
+		unsigned int sflags =
+		    ENABLED_OPT(LOCAL_DNS) ? 0 :
+		    DANE_F_IGNORE_LOCAL_RESOLVER;
+
+		port = service_to_port(service);
+		rc = dane_verify_session_crt(NULL, session, hostname,
+					     udp ? "udp" : "tcp", port,
+					     sflags, 0, &status);
+		if (rc < 0) {
+			fprintf(stderr,
+				"*** DANE verification error: %s\n",
+				dane_strerror(rc));
+			if (!insecure && !ssh)
+				return -1;
+		} else {
+			gnutls_datum_t out;
+
+			rc = dane_verification_status_print(status, &out,
+							    0);
+			if (rc < 0) {
+				fprintf(stderr, "*** DANE error: %s\n",
+					dane_strerror(rc));
+			} else {
+				fprintf(stderr, "- DANE: %s\n", out.data);
+				gnutls_free(out.data);
+			}
+
+			if (status != 0 && !insecure && !ssh)
+				return -1;
+		}
+
+	}
+#endif
+
 	if (ssh) {		/* try ssh auth */
 		unsigned int list_size;
 		const gnutls_datum_t *cert;
@@ -464,7 +507,6 @@ static int cert_verify_callback(gnutls_session_t session)
 						 txt_service,
 						 GNUTLS_CRT_X509, cert, 0);
 		if (rc == GNUTLS_E_NO_CERTIFICATE_FOUND) {
-			print_cert_info_compact(session);
 			fprintf(stderr,
 				"Host %s (%s) has never been contacted before.\n",
 				hostname, txt_service);
@@ -478,7 +520,6 @@ static int cert_verify_callback(gnutls_session_t session)
 			if (rc == 0)
 				return -1;
 		} else if (rc == GNUTLS_E_CERTIFICATE_KEY_MISMATCH) {
-			print_cert_info_compact(session);
 			fprintf(stderr,
 				"Warning: host %s is known and it is associated with a different key.\n",
 				hostname);
@@ -489,10 +530,13 @@ static int cert_verify_callback(gnutls_session_t session)
 					"Its certificate is valid for %s.\n",
 					hostname);
 
-			rc = read_yesno
-			    ("Do you trust the received key? (y/N): ");
-			if (rc == 0)
-				return -1;
+			if (strictssh == 0) {
+				rc = read_yesno
+					("Do you trust the received key? (y/N): ");
+				if (rc == 0)
+					return -1;
+			} else return -1;
+
 		} else if (rc < 0) {
 			fprintf(stderr,
 				"gnutls_verify_stored_pubkey: %s\n",
@@ -511,41 +555,6 @@ static int cert_verify_callback(gnutls_session_t session)
 					gnutls_strerror(rc));
 		}
 	}
-#ifdef HAVE_DANE
-	if (dane) {		/* try DANE auth */
-		int port;
-		unsigned int sflags =
-		    ENABLED_OPT(LOCAL_DNS) ? 0 :
-		    DANE_F_IGNORE_LOCAL_RESOLVER;
-
-		port = service_to_port(service);
-		rc = dane_verify_session_crt(NULL, session, hostname,
-					     udp ? "udp" : "tcp", port,
-					     sflags, 0, &status);
-		if (rc < 0) {
-			fprintf(stderr,
-				"*** DANE verification error: %s\n",
-				dane_strerror(rc));
-			if (!insecure)
-				return -1;
-		} else {
-			gnutls_datum_t out;
-
-			rc = dane_verification_status_print(status, &out,
-							    0);
-			if (rc < 0) {
-				fprintf(stderr, "*** DANE error: %s\n",
-					dane_strerror(rc));
-				if (!insecure)
-					return -1;
-			}
-
-			fprintf(stderr, "- DANE: %s\n", out.data);
-			gnutls_free(out.data);
-		}
-
-	}
-#endif
 
 	return 0;
 }
@@ -1710,6 +1719,8 @@ static int cert_verify_ocsp(gnutls_session_t session)
 	unsigned int cert_list_size = 0;
 	int deinit_issuer = 0;
 	gnutls_datum_t resp;
+	unsigned char noncebuf[23];
+	gnutls_datum_t nonce = { noncebuf, sizeof(noncebuf) };
 	int ret;
 
 	cert_list = gnutls_certificate_get_peers(session, &cert_list_size);
@@ -1746,7 +1757,16 @@ static int cert_verify_ocsp(gnutls_session_t session)
 		goto cleanup;
 	}
 
-	ret = send_ocsp_request(NULL, crt, issuer, &resp, 1);
+	ret =
+	    gnutls_rnd(GNUTLS_RND_NONCE, nonce.data, nonce.size);
+	if (ret < 0) {
+		fprintf(stderr, "gnutls_rnd: %s",
+			gnutls_strerror(ret));
+		ret = -1;
+		goto cleanup;
+	}
+
+	ret = send_ocsp_request(NULL, crt, issuer, &resp, &nonce);
 	if (ret < 0) {
 		fprintf(stderr, "Cannot contact OCSP server\n");
 		ret = -1;
@@ -1754,7 +1774,7 @@ static int cert_verify_ocsp(gnutls_session_t session)
 	}
 
 	/* verify and check the response for revoked cert */
-	ret = check_ocsp_response(crt, issuer, &resp);
+	ret = check_ocsp_response(crt, issuer, &resp, &nonce);
 
       cleanup:
 	if (deinit_issuer)
